


#define BMP085_R 0xEF // Write address
#define BMP085_W 0xEE // Read address
#define OSS 3 // Oversampling Setting

#include "i2c_avr.h"
#include <math.h>
signed short ac1;
signed short ac2;
signed short ac3;
signed short b1;
signed short b2;
signed short mb;
signed short mc;
signed short md;
unsigned short ac4;
unsigned short ac5;
unsigned short ac6;


long bmp085Read2Bytes(unsigned char address)
{
  
	unsigned char msb=0;
	unsigned char lsb=0;
	long data;

  

   delay_ms(10);

   I2CStart();           //  ""   i2c
   I2CWriteByte(BMP085_W);   //    
   I2CWriteByte(address);    //     
   I2CStart();           //   ""  
   I2CWriteByte(BMP085_R);   //  (BMP085) +  
   I2CReadByte(&msb, 0);       //       
   I2CStop(); 
  
   delay_ms(10);

   I2CStart();
   I2CWriteByte(BMP085_W); // Write required to clock in address
   I2CWriteByte(address+1); // Mem address
   I2CStart(); // Restart
   I2CWriteByte(BMP085_R); // Device address for reading
   I2CReadByte(&lsb,0); // Read address
   I2CStop(); // Stop
   delay_ms(10);
   
   data = msb;
   data<<= 8;
   data|=lsb;

   return data;
}

long bmp085Read3Bytes(unsigned char address)
{
    unsigned char msb=0;
	unsigned char lsb=0;
	unsigned char xlsb=0;
	long int data;


   delay_ms(20);

   I2CStart();
   I2CWriteByte(BMP085_W); // Escribir la direccion del sensor
   I2CWriteByte(address); // Direccion de memoria
   I2CStart();            // Reinicio
   I2CWriteByte(BMP085_R); // Direccion para leer del sensor
   I2CReadByte(&msb, 0);     // Leer la direccion
   I2CStop();         //  Stop
   delay_ms(20);


 
   I2CStart();
   I2CWriteByte(BMP085_W); // Escribir la direccion del sensor
   I2CWriteByte(address+1); // Direccion de memoria
   I2CStart();            // Reinicio
   I2CWriteByte(BMP085_R); // Direccion para leer del sensor
   I2CReadByte(&lsb, 0);     // Leer la direccion
   I2CStop();         //  Stop
   delay_ms(20);
  
      
   I2CStart();
   I2CWriteByte(BMP085_W); // Escribir la direccion del sensor
   I2CWriteByte(address+2); // Direccion de memoria
   I2CStart();            // Reinicio
   I2CWriteByte(BMP085_R); // Direccion para leer del sensor
   I2CReadByte(&xlsb, 0);     // Leer la direccion
   I2CStop();         //  Stop
   delay_ms(20);
 
   data = msb;
   data<<= 8;
   data|=lsb;
   data<<=8;
   data|=xlsb;
   data>>=(8-OSS);
 

 
   return data;
}

void BMP085_Calibration(void)
{
  
   ac1 = bmp085Read2Bytes(0xAA);
   ac2 = bmp085Read2Bytes(0xAC);
   ac3 = bmp085Read2Bytes(0xAE);
   ac4 = bmp085Read2Bytes(0xB0);
   ac5 = bmp085Read2Bytes(0xB2);
   ac6 = bmp085Read2Bytes(0xB4);
   b1 = bmp085Read2Bytes(0xB6);
   b2 = bmp085Read2Bytes(0xB8);
   mb = bmp085Read2Bytes(0xBA);
   mc = bmp085Read2Bytes(0xBC);
   md = bmp085Read2Bytes(0xBE);
   
}



long bmp085ReadTemp(void)
{
   I2CStart();

   I2CWriteByte(BMP085_W); 
   I2CWriteByte(0xF4);
   I2CWriteByte(0x2E);
   delay_ms(10); 
   I2CStop();

   delay_ms(10); 

   return bmp085Read2Bytes(0xF6);
}


long bmp085ReadPressure(void)
{
   I2CStart();
   I2CWriteByte(BMP085_W); 
   I2CWriteByte(0xF4);                            //  
   I2CWriteByte(0x34 | (OSS << 6));
   I2CStop();
   
   delay_ms(2 + (3<<OSS));
   return bmp085Read3Bytes(0xF6);

}


void bmp085Convert(unsigned long *pressure,long *temperature)
{
 long  x1, x2, x3, b3, b5 ,b6, ut, up, p;
unsigned long b4, b7;

   ut = bmp085ReadTemp();
   up = bmp085ReadPressure();
  
x1 = (((long) ut - (long) ac6) * (long) ac5) >> 15;
x2 = ((long) mc << 11) / (x1 + md);
b5 = x1 + x2;
*temperature = ((b5 + 8) >> 4); 


b6 = b5 - 4000;
x1 = (b2 * ((b6 * b6) >> 12)) >> 11;
x2 = (ac2 * b6) >> 11;
x3 = x1 +x2;
b3 = ((((long)ac1 *4 + x3) <<OSS) + 2) >> 2;
x1 = (ac3* b6) >> 13;
x2 = (b1 * ((b6*b6) >> 12) ) >> 16;
x3 = ((x1 + x2) + 2) >> 2;
b4 = (ac4 * (unsigned long) (x3 + 32768)) >> 15;
b7 = ((unsigned long)up - b3) * (50000>>OSS);   
if (b7 < 0x80000000)
    p = (b7 << 1) / b4;
else
    p = (b7 / b4) << 1;

x1 = (p >> 8)*(p >> 8);
x1 = (x1 * 3038) >> 16;
x2 = (-7357 * p) >> 16;

*pressure= p + ((x1 + x2 + 3791) >> 4); //press in Pa


//*pressure/=  1333;        // press in mm Hg   //   133.322

}

float bmp085_altitude(uint32_t *pressure)
{
    float temp;
	float  altitude;
	temp = (float) *pressure/101325;
	temp = 1-pow(temp, 0.19029);
	//altitude = round(44330*temp*10);
	altitude = 44330*temp;
	//get altitude in dm
	return altitude;
}