/*******************************************************
This program was created by the
CodeWizardAVR V3.12 Advanced
Automatic Program Generator
 Copyright 1998-2014 Pavel Haiduc, HP InfoTech s.r.l.
http://www.hpinfotech.com

Project : 
Version : 
Date    : 04.01.2016
Author  : 
Company : 
Comments: 


Chip type               : ATmega8
Program type            : Application
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*******************************************************/

#include <mega8.h>
#include <delay.h>
#include <stdio.h>
#include <stdint.h>

// Alphanumeric LCD functions
#include <alcd.h>
//#include <1wire.h>
// DS1820 Temperature Sensor functions
#include <ds18b20.h>

#include "BMP085.c"

// Declare your global variables here
// Voltage Reference: AVCC pin
#define ADC_VREF_TYPE ((0<<REFS1) | (1<<REFS0) | (0<<ADLAR))

#define MAX_DS1820 2 // number of DS1820 devices
                    // connected to the 1 Wire bus       
                    
#define FormatString "t%u=%i.%04u\xdf C\n"
unsigned char ds1820_devices;
// DS1820 devices ROM code storage area,
// 9 bytes are used for each device
// (see the w1_search function description in the help)
unsigned char ds1820_rom_codes[MAX_DS1820][9];
char lcd_buffer[33];

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input | ADC_VREF_TYPE;
// Delay needed for the stabilization of the ADC input voltage
delay_us(10);
// Start the AD conversion
ADCSRA|=(1<<ADSC);
// Wait for the AD conversion to complete
while ((ADCSRA & (1<<ADIF))==0);
ADCSRA|=(1<<ADIF);
return ADCW;
}


//routine for temp measuring
float read_temp(unsigned char *device)
{
    float temp;

    temp=ds18b20_temperature(device);  //   
    if (temp>1000){                          //    1000
        temp=4096.0-temp;                        //   4096
        temp=-temp;
        }
    return temp;
}

void main(void)
{
unsigned char i,j;
long temp;
long UV;
long temp_bmp;
uint32_t press_bmp;
long altitude;

// Alphanumeric LCD initialization
// Connections are specified in the
// Project|Configure|C Compiler|Libraries|Alphanumeric LCD menu:
// RS - PORTB Bit 7
// RD - PORTB Bit 1
// EN - PORTB Bit 2
// D4 - PORTB Bit 3
// D5 - PORTB Bit 4
// D6 - PORTB Bit 5
// D7 - PORTB Bit 6
// Characters/line: 16
lcd_init(16);

// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AVCC pin
ADMUX=ADC_VREF_TYPE;
ADCSRA=(1<<ADEN) | (0<<ADSC) | (0<<ADFR) | (0<<ADIF) | (0<<ADIE) | (1<<ADPS2) | (1<<ADPS1) | (0<<ADPS0);
SFIOR=(0<<ACME);


// 1 Wire Bus initialization
// 1 Wire Data port: PORTB
// 1 Wire Data bit: 0
// Note: 1 Wire port settings are specified in the
// Project|Configure|C Compiler|Libraries|1 Wire menu.

w1_init();

I2CInit(8000000, 100000);
BMP085_Calibration();

// Determine the number of DS1820 devices
// connected to the 1 Wire bus
ds1820_devices=w1_search(0xf0,ds1820_rom_codes);

/* configure each DS18B20 device for 12 bit temperature
   measurement resolution */
for (i=0;i<ds1820_devices;)
        if (!ds18b20_init(&ds1820_rom_codes[i++][0],-20,40,DS18B20_12BIT_RES))
           {
           sprintf(lcd_buffer,"Init error for\ndevice #%u",i);
           lcd_clear();
           lcd_puts(lcd_buffer);
           while (1); /* stop here if init error */
           };

lcd_clear();
/* measure and display the temperature(s) */       
while (1)
          {
                  temp=read_temp(&ds1820_rom_codes[0][0])*10000;
                  lcd_clear();                 
                  sprintf(lcd_buffer,FormatString,1,temp/10000,temp%10000);
                  lcd_puts(lcd_buffer);
                  temp=read_temp(&ds1820_rom_codes[1][0])*10000;
                  sprintf(lcd_buffer,FormatString,2,temp/10000,temp%10000);
                  lcd_puts(lcd_buffer);    
                  delay_ms(2000);
                  //UV= (read_adc(0));
                  //UV= (read_adc(0)*5.0/1024.0)*10000;    
                  //sprintf(lcd_buffer,"U=%i",UV);    
                  bmp085Convert(&press_bmp,&temp_bmp);
                  lcd_clear(); 
                  sprintf(lcd_buffer,"t%u=%i.%1u\xdf C\n",3,temp_bmp/10,temp_bmp%10);
                  lcd_puts(lcd_buffer);
                  delay_ms(2000);
                  UV= ((read_adc(0)/1024.0+0.095)/0.009)*10000;
                  sprintf(lcd_buffer,"P1=%i.%04u kPa\n",UV/10000,UV%10000);
                  lcd_clear();
                  lcd_puts(lcd_buffer);
                  sprintf(lcd_buffer,"P2=%i.%03u kPa",press_bmp/1000,press_bmp%1000);
                  lcd_puts(lcd_buffer); 
                  delay_ms(2000);  
                  lcd_clear();
                  altitude=bmp085_altitude(&press_bmp)*1000;
                  sprintf(lcd_buffer,"Alt=%i.%03u m",altitude/1000,altitude%1000); 
                  lcd_puts(lcd_buffer); 
                  delay_ms(2000); 
          };
}
